/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import atomicsets.OrderedLock;

/**
 * Every {@link AbstractWindow window} has a cursor.
 * The cursor can be moved and hidden.
 *
 * @author Thomas Girard
 * @version 1.0.0
 *
 * @see jcurzez.Pen
 */
public final class Cursor implements atomicsets.Atomic {
	
	/*atomicset(C)*/
	
    public final OrderedLock getLockForC() {
		return $lock_C;
	}

	public final OrderedLock getLock() {
		return this.getLockForC();
	}

	/** The rectangle this cursor is attached to. */
    final Rectangle rectangle;

    /** visibility of the cursor. */
    private int visibility = NORMAL;

    /** x coordinate of the cursor. */
    /*atomic(C)*/ volatile int x;

    /** y coordinate of the cursor. */
    /*atomic(C)*/ volatile int y;
    


    /**
	 * Constructs a new cursor for specified <code>rectangle</code>, with initial position <code>(initialX, initialY)</code>.
	 * @param rectangle  the rectangle this cursor is attached to.
	 * @param initialX   the initial x position of the cursor.
	 * @param initialY   the initial y position of the cursor.
	 */
	public Cursor(Rectangle rectangle, int initialX, int initialY) {
		this(rectangle, initialX, initialY, new OrderedLock());
	}

	/**
     * Constructs a new cursor for specified <code>rectangle</code>,
     * with initial position <code>(initialX, initialY)</code>.
     *
     * @param     rectangle the rectangle this cursor is attached to.
     * @param     initialX  the initial x position of the cursor.
     * @param     initialY  the initial y position of the cursor.
     */
    public Cursor(Rectangle rectangle, int initialX, int initialY, OrderedLock C) {
	$lock_C = C;
	this.rectangle = rectangle;
	this.x = initialX;
	this.y = initialY;
    }

    /**
     * Retrieves the current X coordinate of the cursor.
     *
     * @return    the current X coordinate of the cursor.
     */
    public int getX_internal() {
	return x;
    }

	/**
	 * Retrieves the current X coordinate of the cursor.
	 * @return     the current X coordinate of the cursor.
	 */
	public int getX() {
		synchronized (this.$lock_C) {
			return x;
		}
	}

    /**
     * Retrieves the current Y coordinate of the cursor.
     *
     * @return    the current Y coordinate of the cursor.
     */
    public int getY_internal() {
	return y;
    }

	/**
	 * Retrieves the current Y coordinate of the cursor.
	 * @return     the current Y coordinate of the cursor.
	 */
	public int getY() {
		synchronized (this.$lock_C) {
			return y;
		}
	}

    /**
     * Changes the current X coordinate of the cursor.
     *
     * @param     x the new X coordinate of the cursor.
     *
     * @exception InconsistentCoordinateException if <code>x</code> is not
     *            consistent.
     */
    public void setX_internal(int x) {
	setXY_internal(x, this.y);
    }

	/**
	 * Changes the current X coordinate of the cursor.
	 * @param x  the new X coordinate of the cursor.
	 * @exception InconsistentCoordinateException  if <code>x</code> is not consistent.
	 */
	public void setX(int x) {
		synchronized (this.$lock_C) {
			setXY_internal(x, this.y);
		}
	}

    /**
     * Changes the current Y coordinate of the cursor.
     *
     * @exception InconsistentCoordinateException if
     *            <code>y</code> is not consistent.
     * @param     y the new Y coordinate of the cursor.
     */
    public void setY_internal(int y) {
	setXY_internal(this.x, y);
    }

	/**
	 * Changes the current Y coordinate of the cursor.
	 * @exception InconsistentCoordinateException  if <code>y</code> is not consistent.
	 * @param y  the new Y coordinate of the cursor.
	 */
	public void setY(int y) {
		synchronized (this.$lock_C) {
			setXY_internal(this.x, y);
		}
	}

    /**
     * Changes both the current X and current Y coordinates of the cursor.
     *
     * @param     x the new X coordinate of the cursor.
     * @param     y the new Y coordinate of the cursor.
     *
     * @exception InconsistentCoordinateException if <code>x</code> or
     *            <code>y</code> is not consistent.
     */
    public /*synchronized*/ void setXY_internal(int x, int y) {
	int width = rectangle.getWidth();
	if (x < 0 || x >= width) {
	    throw new
		InconsistentCoordinateException("x: " + x + " is not " +
						"consistent with width:" +
						width);
	}

	int height = rectangle.getHeight();
	if (y < 0 || y >= height) {
	    throw new
		InconsistentCoordinateException("y: " + y + " is not " +
						"consistent with height:" +
						height);
	}
	this.x = x;
	this.y = y;
    }

	/**
	 * Changes both the current X and current Y coordinates of the cursor.
	 * @param x  the new X coordinate of the cursor.
	 * @param y  the new Y coordinate of the cursor.
	 * @exception InconsistentCoordinateException  if <code>x</code> or <code>y</code> is not consistent.
	 */
	public void setXY(int x, int y) {
		synchronized (this.$lock_C) {
			int width = rectangle.getWidth();
			if (x < 0 || x >= width) {
			    throw new
				InconsistentCoordinateException("x: " + x + " is not " +
								"consistent with width:" +
								width);
			}
			int height = rectangle.getHeight();
			if (y < 0 || y >= height) {
			    throw new
				InconsistentCoordinateException("y: " + y + " is not " +
								"consistent with height:" +
								height);
			}
			this.x = x;
			this.y = y;
		}
	}
    
    public int[] getXY_internal(){
    	return new int[]{x,y};
    }

	public int[] getXY() {
		synchronized (this.$lock_C) {
			return new int[]{x,y};
		}
	}

    /**
     * Changes cursor visibility.
     * @param     visible the cursor visibility.
     *
     * @exception IllegalArgumentException if <code>visible</code> is not
     *            consistent.
     */
    public void setCursorVisibility_internal(int visible) {
	if (visible < INVISIBLE || visible > VERY_VISIBLE) {
	    throw new IllegalArgumentException("visible: " + visible +
					       "should be between " +
					       INVISIBLE + " and " +
					       VERY_VISIBLE);
	}
	this.visibility = visible;
//	Screen.getScreen().setCursorVisibility(visible);
    }

	/**
	 * Changes cursor visibility.
	 * @param visible  the cursor visibility.
	 * @exception IllegalArgumentException  if <code>visible</code> is not consistent.
	 */
	public void setCursorVisibility(int visible) {
	if (visible < INVISIBLE || visible > VERY_VISIBLE) {
	    throw new IllegalArgumentException("visible: " + visible +
					       "should be between " +
					       INVISIBLE + " and " +
					       VERY_VISIBLE);
	}
	this.visibility = visible;
//	Screen.getScreen().setCursorVisibility(visible);
    }

    /**
     * Retrieves the cursor visibility.
     *
     * @return the cursor visibility.
     */
    public int getCursorVisibility_internal() {
	return visibility;
    }

	/**
	 * Retrieves the cursor visibility.
	 * @return  the cursor visibility.
	 */
	public int getCursorVisibility() {
	return visibility;
    }

    /** Cursor is not visible. */
    public static final int INVISIBLE = 0;

    /** Cursor is visible. */
    public static final int NORMAL = 1;

    /** Cursor is very visible. */
    public static final int VERY_VISIBLE = 2;

	protected final OrderedLock $lock_C;
}
