/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import atomicsets.OrderedLock;

/**
 * A rectangle is defined by its bounds ({@link #top}, {@link #left},
 * {@link #right}, {@link #bottom}) a {@link #fillingPen background pen},
 * and a {@link #drawingPen drawing pen}.
 * The first pen will be used to fill in the rectangle when new
 * room is created. This is a kind of background pen. The second
 * one is the one you should change when you want to write a new thing
 * on the window, using different colors and/or attributes.
 * <p>
 * This class doesn't provide drawing primitives, since drawing is
 * delegated to {@link jcurzez.AbstractWindow AbstractWindow}.
 *
 * @author Thomas Girard
 * @version 1.0.0
 *
 * @see jcurzez.BackgroundColor
 */
public class Rectangle implements atomicsets.Atomic {
    //Those fields are not final because maybe a rectangle could be resized or
    //moved.
	
	/*atomicset(R)*/

    public final OrderedLock getLockForR() {
		return $lock_R;
	}

	public final OrderedLock getLock() {
		return this.getLockForR();
	}

	/** top coordinate of this rectangle. */
    /*atomic(R)*/ protected int top;

    /** left coordinate of this rectangle. */
    /*atomic(R)*/ protected int left;

    final protected int width;
    final protected int height;
    
//    /** right coordinate of this rectangle. */
//    protected int right;
//
//    /** bottom coordinate of this rectangle. */
//    protected int bottom;

    /** pen used to fill in this rectangle.*/
    protected final FillingPen fillingPen;

    /** pen used to draw new cells in this rectangle. */
    protected final Pen drawingPen;

    /**
	 * Creates a new Rectangle with the specified bounds, and a default pen for drawing and filling.
	 * @param left  left coordinate.
	 * @param top  top coordinate.
	 * @param right  right coordinate.
	 * @param bottom  bottom coordinate.
	 */
	public Rectangle(int left, int top, int right, int bottom) {
		this(left, top, right, bottom, new OrderedLock());
	}

	/**
     * Creates a new Rectangle with the specified bounds,
     * and a default pen for drawing and filling.
     *
     * @param left left coordinate.
     * @param top top coordinate.
     * @param right right coordinate.
     * @param bottom bottom coordinate.
     */
    public Rectangle(int left, int top, int right, int bottom, OrderedLock R) {
	this(left, top, right, bottom, new Pen(), new Pen(), R);
    }

    /**
	 * Creates a new Rectangle with the specified bounds, specified pen fo filling, and a default pen for drawing.
	 * @param left  left coordinate.
	 * @param top  top coordinate.
	 * @param right  right coordinate.
	 * @param bottom  bottom coordinate.
	 * @param fillingPen  the pen used to fill cells.
	 */
	public Rectangle(int left, int top, int right, int bottom, Pen fillingPen) {
		this(left, top, right, bottom, fillingPen, new OrderedLock());
	}

	/**
     * Creates a new Rectangle with the specified bounds,
     * specified pen fo filling, and a default pen for drawing.
     *
     * @param left left coordinate.
     * @param top top coordinate.
     * @param right right coordinate.
     * @param bottom bottom coordinate.
     * @param fillingPen the pen used to fill cells.
     */
    public Rectangle(int left, int top, int right, int bottom,
		     Pen fillingPen, OrderedLock R) {
	this(left, top, right, bottom, fillingPen, new Pen(), R);
    }

    /**
	 * Creates a new Rectangle with the specified bounds, and specified background color.
	 * @param left  left coordinate.
	 * @param top  top coordinate.
	 * @param right  right coordinate.
	 * @param bottom  bottom coordinate.
	 * @param fillingPen  the pen to fill the rectangle.
	 * @param drawingPen  the pen to draw new cells.
	 */
	public Rectangle(int left, int top, int right, int bottom, Pen fillingPen,
			Pen drawingPen) {
		this(left, top, right, bottom, fillingPen, drawingPen,
				new OrderedLock());
	}

	/**
     * Creates a new Rectangle with the specified bounds, and specified
     * background color.
     *
     * @param left left coordinate.
     * @param top top coordinate.
     * @param right right coordinate.
     * @param bottom bottom coordinate.
     * @param fillingPen the pen to fill the rectangle.
     * @param drawingPen the pen to draw new cells.
     */
    public Rectangle(int left, int top, int right, int bottom,
		     Pen fillingPen, Pen drawingPen, OrderedLock R) {
	$lock_R = R;
	this.top = top;
	this.left = left;
	this.height = bottom-top;
	this.width = right-left;
	this.fillingPen = new FillingPen(fillingPen);
	this.drawingPen = drawingPen;
    }
    
    
    public /*synchronized*/ int[] getTopLeft_internal(){
    	return new int[]{top,left};
    }

	public int[] getTopLeft() {
		synchronized (this.$lock_R) {
			return new int[]{top,left};
		}
	}
    
    public int[] getTopLeftBottomRight_internal(){
    	return new int[] {top,left,getBottom_internal(),getRight_internal()};
    }

	public int[] getTopLeftBottomRight() {
		synchronized (this.$lock_R) {
			return new int[] {top,left,getBottom_internal(),getRight_internal()};
		}
	}
    
    public boolean contains_internal(/*unitfor(R)*/Rectangle child){
    	int[] me = getTopLeftBottomRight_internal();
    	int[] you = child.getTopLeftBottomRight_internal();
    	
    	return me[0] <= you[0] && me[1] <= you[1] && me[2] >= you[2] && me[3] >= you[3];
    }

	public boolean contains(Rectangle child) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = child.getLockForR();
			OrderedLock l4 = this.$lock_R;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				int[] me = getTopLeftBottomRight_internal();
				int[] you = child.getTopLeftBottomRight_internal();
				return me[0] <= you[0] && me[1] <= you[1] && me[2] >= you[2] && me[3] >= you[3];
			}
		}
	}
    
    public void move_internal(int dx, int dy, /*unitfor(R)*/ Rectangle parent){
    	left += dx;
    	top += dy;
    	if (!parent.contains_internal(this)){
    		left -= dx;
    		top -= dy;
    		throw new
			InconsistentCoordinateException("New position would make " +
							"part of the window fall " +
							"outside its parent area");
    	}	
    }

	public void move(int dx, int dy, Rectangle parent) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = parent.getLockForR();
			OrderedLock l4 = this.$lock_R;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				left += dx;
				top += dy;
				if (!parent.contains_internal(this)){
					left -= dx;
					top -= dy;
					throw new
					InconsistentCoordinateException("New position would make " +
									"part of the window fall " +
									"outside its parent area");
				}
			}
		}
	}

    /**
     * Returns the left coordinate.
     * @return the left coordinate.
     */
    public int getLeft_internal() {
	return left;
    }

	/**
	 * Returns the left coordinate.
	 * @return  the left coordinate.
	 */
	public int getLeft() {
		synchronized (this.$lock_R) {
			return left;
		}
	}

    /**
     * Returns the top coordinate.
     * @return the top coordinate.
     */
    public int getTop_internal() {
	return top;
    }

	/**
	 * Returns the top coordinate.
	 * @return  the top coordinate.
	 */
	public int getTop() {
		synchronized (this.$lock_R) {
			return top;
		}
	}

    /**
     * Returns the right coordinate.
     * @return the right coordinate.
     */
    public int getRight_internal() {
	return left+width;
    }

	/**
	 * Returns the right coordinate.
	 * @return  the right coordinate.
	 */
	public int getRight() {
		synchronized (this.$lock_R) {
			return left+width;
		}
	}

    /**
     * Returns the bottom coordinate.
     * @return the bottom coordinate.
     */
    public int getBottom_internal() {
	return top+height;
    }

	/**
	 * Returns the bottom coordinate.
	 * @return  the bottom coordinate.
	 */
	public int getBottom() {
		synchronized (this.$lock_R) {
			return top+height;
		}
	}

    /**
     * Returns the height of the rectangle.
     * @return the height of the rectangle.
     */
    public int getHeight_internal() {
	return height;
    }

	/**
	 * Returns the height of the rectangle.
	 * @return  the height of the rectangle.
	 */
	public int getHeight() {
	return height;
    }

    /**
     * Returns the width of the rectangle.
     * @return the width of the rectangle.
     */
    public int getWidth_internal() {
	return width;
    }

	/**
	 * Returns the width of the rectangle.
	 * @return  the width of the rectangle.
	 */
	public int getWidth() {
	return width;
    }

    /**
     * Returns the drawing pen of this rectangle.
     * @return {@link #drawingPen}.
     */
    public Pen getDrawingPen_internal() {
	return drawingPen;
    }

	/**
	 * Returns the drawing pen of this rectangle.
	 * @return   {@link #drawingPen} .
	 */
	public Pen getDrawingPen() {
	return drawingPen;
    }

    /**
     * Returns the filling pen.
     * @return {@link #fillingPen}.
     */
    public Pen getFillingPen_internal() {
	return fillingPen;
    }

	/**
	 * Returns the filling pen.
	 * @return   {@link #fillingPen} .
	 */
	public Pen getFillingPen() {
	return fillingPen;
    }

    /**
     * Returns a representation of <code>this</code>.
     *
     * @return a representation of <code>this</code>.
     */
    public String toString_internal() {
	StringBuffer sb = new StringBuffer("[left=");
	sb.append(left).append(",top=").append(top).append(",height=");
	sb.append(height).append(",width=").append(width).append(']');
	return sb.toString();
    }

	/**
	 * Returns a representation of <code>this</code>.
	 * @return  a representation of <code>this</code>.
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer("[left=");
		synchronized (this.$lock_R) {
			sb.append(left).append(",top=").append(top).append(",height=");
			sb.append(height).append(",width=").append(width).append(']');
			return sb.toString();
		}
	}

	protected final OrderedLock $lock_R;
}
