/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001-2007  Thomas Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import org.nongnu.savannah.jcurzez.io.JcurzezOutputStream;
import org.nongnu.savannah.jcurzez.peer.NullScreenPeer;
import org.nongnu.savannah.jcurzez.peer.ScreenPeer;

import atomicsets.OrderedLock;
import java.io.PrintStream;

/**
 * This class inherits features from
 * {@link jcurzez.AbstractWindow AbstractWindow}, and internally uses
 * a {@link jcurzez.peer.ScreenPeer peer component} so that it is
 * a concrete class. <br>
 * The peer component is chosen using system properties,
 * a explained {@link #getPeerPackageName here}.
 * <p>
 * <i>Note: To decide if the screen should have linewrapping and autoscrolling
 * features, one may use <code>jcurzez.term.linewrap</code> and
 * <code>jcurzez.term.scroll</code> system properties, which both
 * default to <code>true</code> when not set.</i>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class Screen extends AbstractWindow {
    /** The peer implementation. */
	private static volatile ScreenPeer peer;

    /**
     * <code>true</code> if the screen has the linewrap feature.
     * <p>
     * It is determined using the <code>jcurzez.term.linewrap</code>
     * system propety, and defaults to <code>true</code> if this
     * property is not set.
     */
    private static final boolean screenLinewrap;

    /**
     * <code>true</code> if the screen has the scroll feature.
     * <p>
     * It is determined using the <code>jcurzez.term.scroll</code>
     * system propety, and defaults to <code>true</code> if this
     * property is not set.
     */
    private static final boolean screenScroll;

    /** Previous object for <code>stdout</code>. */
    public static final PrintStream previousOut;

    /** Previous object for <code>stderr</code>. */
    public static final PrintStream previousErr;


    /** The one and only screen. */
    private static Screen SCREEN = null;

    /** The lock on the peer. */
//    /*private*/ static final Object peerLock = new Object();

    /**
     * To achieve screen creation, the <code>Screen</code>
     * static initializer uses the <code>jcurzez.term.type</code> property.
     * <p>
     * If this property is not set we try to guess which implementation
     * we should use with the <code>os.name</code> property, with the
     * following rules:
     * <ul>
     *   <li>If the os contains <code>"LINUX"</code>,
     *       we use <b>curses</b> implementation</li>
     *   <li>If the os contains <code>"WIN"</code>, we use <b>win</b>
     *       implementation</li>
     *   <li>If none of this tests succeeded, implementation defaults to
     *       <b>ansi</b></li>
     * </ul>
     *
     * @return the peer package name.
     */
    private static final String getPeerPackageName() {
	String terminalType = System.getProperty("jcurzez.term.type");
	if (terminalType == null) {
	    String osName = System.getProperty("os.name").toUpperCase();
	    if (osName.indexOf("LINUX") != -1)
		// || osName.indexOf("SUNOS") != -1)
		return "jcurzez.peer.curses";
	    if (osName.indexOf("WIN") != -1)
		return "jcurzez.peer.win";
	    return "jcurzez.peer.ansi";
	} else /* @todo: check domain */
	    return "jcurzez.peer." + terminalType;
    }


    /**
     * Static initializer: tries to instantiate a peer screen. Then
     * retrieves values of {@link #screenScroll} and {@link #screenLinewrap}
     * using system properties. Eventually, a shutdown hook is set to restore
     * everything when the JVM is stopped.
     */
    static {
	try {
	    //Keeps old System.out and System.err
	    previousOut = System.out;
	    previousErr = System.err;

	    //Adds a shutdown hook to restore screen on either program exit
	    // or JVM end. This is JDK 1.3 specific.
	    Runtime.getRuntime().addShutdownHook(new JCurzezHook());

	    peer = (ScreenPeer) Class.forName("org.nongnu.savannah." + getPeerPackageName() +
	      ".PeerScreen").newInstance();
	} catch(ClassNotFoundException cnfe) {
	    throw new RuntimeException(cnfe.getMessage());
	} catch(InstantiationException ie) {
	    throw new RuntimeException(ie.getMessage());
	} catch(IllegalAccessException iae) {
	    throw new RuntimeException(iae.getMessage());
	} catch(ClassCastException cce) {
	    throw new RuntimeException(cce.getMessage());
	}

	//two object allocated but not needed...
	screenLinewrap =
	    Boolean.valueOf(System.getProperty("jcurzez.term.linewrap",
					       "true")).booleanValue();
	screenScroll   =
	    Boolean.valueOf(System.getProperty("jcurzez.term.scroll",
					       "true")).booleanValue();
    }

    /**
	 * Creates a new <code>Screen</code>. <p> A hook is set so that all calls to <code>System.out</code> and <code>System.err</code> are routed to <code>this</code>.
	 */
	private Screen() {
		this(new OrderedLock());
	}

	/**
     * Creates a new <code>Screen</code>.
     * <p>
     * A hook is set so that all calls to <code>System.out</code> and
     * <code>System.err</code> are routed to <code>this</code>.
     */
    private Screen(OrderedLock B) {
	super(null,  /* a screen has no parent. */
	      new Cursor(new Rectangle(0, 0,
				       peer.getColumns(), /* delegated*/
				       peer.getLines(), new OrderedLock()),  /* to peer component*/
			 peer.getFirstX(),                /* initial position*/
			 peer.getFirstY(), new OrderedLock()),               /* of the cursor   */
	      screenLinewrap,                             /* linewrap ? */
	      screenScroll, B);                              /* scroll ?   */

	//Sets out  to his new value.
	PrintStream outAndErr = new PrintStream(new JcurzezOutputStream(this));
	System.setOut(outAndErr);
	System.setErr(outAndErr);
    }

    /**
     * Retrieves the one and only screen.
     *
     * @return the screen.
     */
    public static Screen getScreen() {
	if (SCREEN == null) {
	    SCREEN = new Screen(new OrderedLock());
	}
	return SCREEN;
    }

    /**
     * Delegates call to {@link #peer}.
     */
    public void beep_internal() {
	{ // synchronized (peerLock) {
	    peer.beep();
	}
    }

	/**
	 * Delegates call to  {@link #peer} .
	 */
	public void beep() {
		{
			synchronized (this.$lock_B) {
				peer.beep();
			}
		}
	}

    /**
     * Delegates call to {@link #peer}.
     */
    protected void gotoXY_internal(int x, int y) {
	{ // synchronized (peerLock) {
	    peer.gotoXY(x, y);
	}
    }

	/**
	 * Delegates call to  {@link #peer} .
	 */
	protected void gotoXY(int x, int y) {
		{
			synchronized (this.$lock_B) {
				peer.gotoXY(x, y);
			}
		}
	}

    /**
     * Delegates call to {@link #peer}.
     */
    void setCursorVisibility_internal(int visibility) {
	{ // synchronized (peerLock) {
	    peer.setCursorVisibility(visibility);
	}
    }

	/**
	 * Delegates call to  {@link #peer} .
	 */
	void setCursorVisibility(int visibility) {
		{
			synchronized (this.$lock_B) {
				peer.setCursorVisibility(visibility);
			}
		}
	}

    /**
     * Delegates call to {@link #peer}.
     */
    protected void printCell_internal(int x, int y, final Cell cell) {
	{ // synchronized (peerLock) {
	    peer.printCell(x, y, cell);
	}
    }

	/**
	 * Delegates call to  {@link #peer} .
	 */
	protected void printCell(int x, int y, final Cell cell) {
		{
			synchronized (this.$lock_B) {
				peer.printCell(x, y, cell);
			}
		}
	}

    /**
     * Delegates call to {@link #peer}.
     */
    protected void update_internal() {
	{ // synchronized (peerLock) {
	    peer.update();
	}
    }

	/**
	 * Delegates call to  {@link #peer} .
	 */
	protected void update() {
		{
			synchronized (this.$lock_B) {
				peer.update();
			}
		}
	}

    /**
     * The shutdown hook that will be lauched upon VM termination
     * or program exit.
     */
    private static class JCurzezHook extends Thread {
	JCurzezHook() {
	    super("jcurzez-hook");
	}

	/**
	 * This method is invoked by the shutdown hook.
	 * <p>
	 * The screen is cleared, {@link jcurzez.peer#end}
	 * is invoked, the {@link #peer} is changed to a {@link
	 * jcurzez.peer.NullScreenPeer}.  Eventually <code>System.out</code>
	 * and <code>System.err</code> is restored.
	 */
	public void run() {
	    { // synchronized (peerLock) {
//		int lines = peer.getLines();
//		int columns = peer.getColumns();
//		peer.gotoXY(0, lines - 1);
//		peer.end();
//		peer = new NullScreenPeer(columns, lines);
	    	peer.end();
	    }

	    System.setOut(previousOut);
	    System.setErr(previousErr);
	}
    }
}
