/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.peer.curses;

import atomicsets.OrderedLock;
import org.nongnu.savannah.jcurzez.peer.ScreenPeer;
import org.nongnu.savannah.jcurzez.*;

/**
 * The <b>curses</b> implementation of <b>Jcurzez</b>.
 * Almost all methods are native.
 *
 * @version 1.0.0
 * @author Thomas Girard
 */
public final class PeerScreen implements ScreenPeer, atomicsets.Atomic {

    public final OrderedLock getLockForP() {
		return $lock_P;
	}

	public final OrderedLock getLock() {
		return this.getLockForP();
	}

	static {
	//If an exception is raised,
	//it isn't caught. Program relying on jcurzez will
	//terminate.
	System.loadLibrary("jcz_curses");
    }

    private static final int START_COLOR_ERR = -2;
    private static final int INITSCR_ERR     = -1;
    private static final int INITSCR_OK      =  0;
    private static final int NO_COLORS       =  1;

    private final boolean hasColors;


    private static native int _begin();

    public PeerScreen() {
		this(new OrderedLock());
	}

	public PeerScreen(OrderedLock P) {
	$lock_P = P;
	boolean colors = true;
	int r = _begin();

	//Initializes curses and colors.
	switch(r) {
	case START_COLOR_ERR:
	    throw new RuntimeException("curses start_color() failed, aborting...");
	case INITSCR_ERR:
	    throw new RuntimeException("curses initscr() failed, aborting...");
	case NO_COLORS:
	    colors = false;
	    break;
	case INITSCR_OK:
	    break;
	default:
	    _end();
	    throw new RuntimeException("native call to _begin() returned: " +
				       r + " which is an unknown value, aborting...");
	}

	this.hasColors = colors;
    }

    private static native void _gotoXY(int x, int y);

    public void gotoXY_internal(int x, int y) {
	_gotoXY(x, y);
    }

	public void gotoXY(int x, int y) {
		synchronized (this.$lock_P) {
			_gotoXY(x, y);
		}
	}

    private static native void _printCell(int x, int y,
					  short fore, short back,
					  byte attr, int c);

    /**
     */
    public void printCell_internal(int x, int y, final Cell cell) {
	short f =
	    hasColors ?
	    ((CursesColor) cell.foreground.getRealColor()).index :
	    (short) 0;
	short b =
	    hasColors ?
	    ((CursesColor) cell.background.getRealColor()).index :
	    (short) 0;

	_printCell(x, y, f, b,
		   cell.attribute.getValue(),
		   cell.character);
    }

	/**
	 */
	public void printCell(int x, int y, final Cell cell) {
		synchronized (this.$lock_P) {
			short f =
			    hasColors ?
			    ((CursesColor) cell.foreground.getRealColor()).index :
			    (short) 0;
			short b =
			    hasColors ?
			    ((CursesColor) cell.background.getRealColor()).index :
			    (short) 0;
			_printCell(x, y, f, b,
				   cell.attribute.getValue(),
				   cell.character);
		}
	}

    private static native int _getColumns();

    /**
     */
    public int getColumns_internal() {
	return _getColumns();
    }

	/**
	 */
	public int getColumns() {
		synchronized (this.$lock_P) {
			return _getColumns();
		}
	}

    private static native int _getLines();

    /**
     */
    public int getLines_internal() {
	return _getLines();
    }

	/**
	 */
	public int getLines() {
		synchronized (this.$lock_P) {
			return _getLines();
		}
	}

    /**
     */
    public int getFirstX_internal() {
	//because curses deletes the screen on first invocation...
	return 0;
    }

	/**
	 */
	public int getFirstX() {
	//because curses deletes the screen on first invocation...
	return 0;
    }

    /**
     */
    public int getFirstY_internal() {
	//because curses deletes the screen on first invocation...
	return 0;
    }

	/**
	 */
	public int getFirstY() {
	//because curses deletes the screen on first invocation...
	return 0;
    }

    private static native void _update();

    /**
     */
    public void update_internal() {
	_update();
    }

	/**
	 */
	public void update() {
		synchronized (this.$lock_P) {
			_update();
		}
	}

    private static native void _setCursorVisibility(int v);

    /**
     */
    public void setCursorVisibility_internal(int v) {
	_setCursorVisibility(v);
    }

	/**
	 */
	public void setCursorVisibility(int v) {
		synchronized (this.$lock_P) {
			_setCursorVisibility(v);
		}
	}

    private static native void _beep();

    /**
     */
    public void beep_internal() {
	_beep();
    }

	/**
	 */
	public void beep() {
		synchronized (this.$lock_P) {
			_beep();
		}
	}

    private static native void _end();

    /**
     */
    public void end_internal() {
	//curses shutdown.
	_end();
    }

	/**
	 */
	public void end() {
		synchronized (this.$lock_P) {
			//curses shutdown.
			_end();
		}
	}

	protected final OrderedLock $lock_P;
}
