/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.peer;

import org.nongnu.savannah.jcurzez.*;

/**
 * This interface consists in few methods that a peer class must implement to
 * provide <b>jcurzez</b> a {@link jcurzez.Screen Screen}.
 * <p>
 * The <code>{@link jcurzez.Screen Screen}</code> class relies on this
 * interface, as <code>{@link jcurzez.Area Area}</code> eventually does.
 *
 * @see jcurzez.Screen#peer
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public interface ScreenPeer {
	
	/*atomicset(P)*/
	
    /**
     * Moves the cursor <b>on the physical screen</b> at specified position.
     * This method must not be called directly. <code>x</code> and
     * <code>y</code> should be consistent.
     * This method is used by {@link jcurzez.AbstractWindow#refresh refresh}
     * to move the cursor where it is supposed to be.
     *
     * @param     x x coordinate where to send the cursor. This is an absolute
     *            coordinate.
     * @param     y y coordinate where to send the cursor. This is an absolute
     *            coordinate.
     * @see       jcurzez.AbstractWindow#refresh
     */
    public void gotoXY(int x, int y);

    /**
     * Prints the cell <code>cell</code> <b>on the physical screen</b> at
     * specified position.
     *
     * @param     x x coordinate where to send the cursor. This is an absolute
     *            coordinate.
     * @param     y y coordinate where to send the cursor. This is an absolute
     *            coordinate.
     * @param     cell the cell to print at this position.
     * @see       jcurzez.Cell
     */
    public void printCell(int x, int y, final Cell cell);

    /**
     * Updates the <b>physical screen</b>. This method provides
     * a way for a peer to optimize its display. It is called when
     * all needed cells have been printed, and before the cursor
     * is moved back - using {@link #gotoXY} - to where it should be.
     *
     * @see       jcurzez.AbstractWindow#refresh
     */
    public void update();

    /**
     * Retrieves the number of columns the screen has. This is often
     * <code>80</code>, but it may be different if you use a <code>xterm</code>,
     * a different screen mode, or frame buffering.
     *
     * @return    the number of columns the screen has.
     */
    public int getColumns();

    /**
     * Retrieves the number of lines the screen has. This is often
     * <code>25</code>, but it may be different if you use a <code>xterm</code>,
     * a different screen mode, or frame buffering.
     *
     * @return    the number of lines the screen has.
     */
    public int getLines();

    /**
     * Retrieves the x coordinate by the time the screen object is created.
     *
     * @return    the x coordinate at the beginning.
     */
    public int getFirstX();

    /**
     * Retrieves the y coordinate by the time the screen object is created.
     *
     * @return    the y coordinate at the beginning.
     */
    public int getFirstY();

    /**
     * Changes the way cursor is displayed.
     *
     * @param     v visibility of the cursor. One of
     *            {@link jcurzez.Cursor#NORMAL NORMAL},
     *            {@link jcurzez.Cursor#INVISIBLE INVISIBLE},
     *            {@link jcurzez.Cursor#VERY_VISIBLE VERY_VISIBLE}.
     * @see       jcurzez.Cursor
     */
    public void setCursorVisibility(int v);

    /**
     * Emits an audio beep.
     */
    public void beep();

    /**
     * Performs all shutdown tasks before the screen is destroyed.
     */
    public void end();
}
