/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001-2008 Thomas Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.peer.ansi;

import org.nongnu.savannah.jcurzez.peer.ScreenPeer;
import org.nongnu.savannah.jcurzez.*;

/**
 * This is the <b>ansi</b> implementation of <b>jcurzez</b>.
 * There are no native calls, thus this implementation should work on
 * any platform that supports Java and ansi.
 * <p>
 * For retrieving size of the screen, we use four system properties:
 * <ul>
 *   <li><code>jcurzez.term.columns</code></li>
 *   <li><code>jcurzez.term.lines</code></li>
 *   <li><code>jcurzez.term.x</code></li>
 *   <li><code>jcurzez.term.y</code></li>
 * </ul>
 * <p>
 * Those properties can be set on JVM invocation using <code>tput(1)</code>,
 * or environment variables <code>COLUMNS</code>
 * and <code>LINES</code>. If they are not set, they default to
 * <code>80</code>, <code>24</code> (or 25 for Windows) and <code>0</code>,
 * <code>0</code>.
 *
 * @version 1.0.0
 * @author Thomas Girard
 */
public final class PeerScreen implements ScreenPeer {
    /** Number of columns the screen has. */
    /*atomic(P)*/ private int columns;

    /** Number of lines the screen has. */
    /*atomic(P)*/ private int lines;

    /** To optimize cursor motion, we store previous cursor position. */
    /*atomic(P)*/ private int previousX;

    /** To optimize cursor motion, we store previous cursor position. */
    /*atomic(P)*/ private int previousY;

    /** Current representation of the content of the screen. */
    /*atomic(P)*/ private StringBuffer text;

    /**
     * To optimize, we keep the last cell. If the next adjacent cell
     * has the same decoration, we won't send escape sequences
     * again.
     */
    /*atomic(P)*/ private Cell previousCell;
    
    /*atomic(P)*/ private boolean completed;

    /*private static final int[] retrieveXAndY() {
	//Try to retrieve term.x, term.y using escape sequences
	System.out.print("\033[6n");
      }*/

    /**
     * Constructs a new ANSI PeerScreen. Then sets
     * colors' real objects (using {@link jcurzez.Color#setRealColor})
     * which, for the <b>ansi</b> implementation, are escape sequences.
     */
    public PeerScreen() {
	//Initializes size of the screen and initial
	//position of the cursor according to system properties.
	this.columns      =
	    Integer.getInteger("jcurzez.term.columns", 80).intValue();

	//For windows, lines defaults to 25, otherwise 24.
	int defaultLines = 24;
	if (System.getProperty("os.name").toUpperCase().indexOf("WIN") != -1) {
	    defaultLines = 25;
	}

	this.lines        =
	    Integer.getInteger("jcurzez.term.lines", defaultLines).intValue();
	this.previousX    =
	    Integer.getInteger("jcurzez.term.x", 0).intValue();
	this.previousY    =
	    Integer.getInteger("jcurzez.term.y", 0).intValue();

	this.text         = new StringBuffer(columns * lines);
	this.previousCell = Cell.NULL_CELL;

	//Initializes colors.
	ForegroundColor.BLACK.setRealColor("30");
	ForegroundColor.RED.setRealColor("31");
	ForegroundColor.GREEN.setRealColor("32");
	ForegroundColor.YELLOW.setRealColor("33");
	ForegroundColor.BLUE.setRealColor("34");
	ForegroundColor.MAGENTA.setRealColor("35");
	ForegroundColor.CYAN.setRealColor("36");
	ForegroundColor.WHITE.setRealColor("37");

	BackgroundColor.BLACK.setRealColor("40");
	BackgroundColor.RED.setRealColor("41");
	BackgroundColor.GREEN.setRealColor("42");
	BackgroundColor.YELLOW.setRealColor("43");
	BackgroundColor.BLUE.setRealColor("44");
	BackgroundColor.MAGENTA.setRealColor("45");
	BackgroundColor.CYAN.setRealColor("46");
	BackgroundColor.WHITE.setRealColor("47");
    }

    /**
     */
    public void gotoXY(int x, int y) {
    	if (completed) return;
	Screen.previousOut.print("\033[" + (y + 1) + ";" + (x + 1) + "H");
    }

    /**
     * Computes the representation of the attribute <code>a</code>,
     * and appends this representation to {@link #text}.
     *
     * @param     a the attribute we want to use.
     */
    private void attributeToEscapeSequence(Attribute a) {
    	if (completed) return;
	//Resets all existing attributes.
	    text.append("0;");

	if (a.hasAttribute(Attribute.BLINK)) {
	    text.append("5;");
	}

	if (a.hasAttribute(Attribute.BOLD)) {
	    text.append("1;");
	}

	if (a.hasAttribute(Attribute.REVERSE)) {
	    text.append("7;");
	}

	if (a.hasAttribute(Attribute.UNDERLINE)) {
	    text.append("4;");
	}
    }

    /**
     */
    public void printCell(int x, int y, final Cell cell) {
    	if (completed) return;
	if (!( ((x == previousX + 1) && (y == previousY)) ||
	       ((x == 0) && (previousX == columns - 1) && (y == previousY + 1)))) {
	    text.append("\033[" + (y + 1) + ";" + (x + 1) + "H");
	}
	if (!cell.hasSameDecoration(previousCell)) {
	    text.append("\033[");
	    attributeToEscapeSequence(cell.attribute);
	    text.append((String) cell.foreground.getRealColor());
	    text.append(';');
	    text.append((String) cell.background.getRealColor());
	    text.append('m');
	}

	final char c = (char) cell.character;
	switch(c) {
	case BoxDrawing.LIGHT_HORIZONTAL:
	case BoxDrawing.DOUBLE_HORIZONTAL:
	case BoxDrawing.HEAVY_HORIZONTAL:
	    text.append("\033(0q\033(B");
	    break;
	case BoxDrawing.LIGHT_VERTICAL:
	case BoxDrawing.DOUBLE_VERTICAL:
	case BoxDrawing.HEAVY_VERTICAL:
	    text.append("\033(0x\033(B");
	    break;
	case BoxDrawing.LIGHT_DOWN_AND_RIGHT:
	case BoxDrawing.DOWN_LIGHT_AND_RIGHT_HEAVY:
	case BoxDrawing.DOWN_HEAVY_AND_RIGHT_LIGHT:
	case BoxDrawing.HEAVY_DOWN_AND_RIGHT:
	    text.append("\033(0l\033(B");
	    break;
	case BoxDrawing.LIGHT_UP_AND_RIGHT:
	case BoxDrawing.UP_LIGHT_AND_RIGHT_HEAVY:
	case BoxDrawing.UP_HEAVY_AND_RIGHT_LIGHT:
	case BoxDrawing.HEAVY_UP_AND_RIGHT:
	    text.append("\033(0m\033(B");
	    break;
	case BoxDrawing.LIGHT_UP_AND_LEFT:
	case BoxDrawing.UP_LIGHT_AND_LEFT_HEAVY:
	case BoxDrawing.UP_HEAVY_AND_LEFT_LIGHT:
	case BoxDrawing.HEAVY_UP_AND_LEFT:
	    text.append("\033(0j\033(B");
	    break;
	case BoxDrawing.LIGHT_DOWN_AND_LEFT:
	case BoxDrawing.DOWN_LIGHT_AND_LEFT_HEAVY:
	case BoxDrawing.DOWN_HEAVY_AND_LEFT_LIGHT:
	case BoxDrawing.HEAVY_DOWN_AND_LEFT:
	    text.append("\033(0k\033(B");
	    break;
	default:
	    text.append(c);
	}

	previousX    = x;
	previousY    = y;
	previousCell = cell;
    }

    /**
     * Relies on the <code>jcurzez.term.columns</code> property,
     * or defaults to <code>80</code> if this property is not set.
     */
    public int getColumns() {
	return columns;
    }

    /**
     */
    public int getLines() {
	return lines;
    }

    /**
     */
    public int getFirstX() {
    	if (completed) return 0;
	return previousX;
    }

    /**
     */
    public int getFirstY() {
    	if (completed) return 0;
	return previousY;
    }

    /**
     */
    //Always protected by external lock anyway...
    public /*synchronized*/ void update() {
    	if (completed) return;
	//Sends the representation to the screen.
	Screen.previousOut.print(text.toString());
	Screen.previousOut.flush();

	//Resets everything.
	text.setLength(0);
	previousCell = Cell.NULL_CELL;
	previousY    = -1;
	previousX    = -1;
    }

    /**
     * Does nothing for the ansi implementation.
     */
    public void setCursorVisibility(int v) {
	//It looks like the cursor can't be hidden using ANSI.
	//Maybe I'm wrong ?
    }

    /**
     */
    public void beep() {

    	if (completed) return;
	//Emits an audio beep. Depending on your terminal configuration,
	//the screen can keep silent.
	Screen.previousOut.print('\007');
    }

    /**
     * Restores default color.
     */
    public void end() {
    	if (completed) return;
    	
    	gotoXY(0, getLines()-1);
    	completed=true;
	Screen.previousOut.print("\033[0;m");
    }
}
