/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001-2007  Thomas Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"
#include "jcurzez_curses.h"

#ifdef HAVE_CURSES_H
#include <curses.h>
#endif
#ifdef HAVE_NCURSES_H
#include <ncurses.h>
#endif

#include "../jcurzez_BoxDrawing.h"

#define C_PEN             "jcurzez/Pen"
#define C_COLOR           "jcurzez/Color"
#define C_CURSESCOLOR     "jcurzez/peer/curses/CursesColor"
#define C_FOREGROUNDCOLOR "jcurzez/ForegroundColor"
#define C_BACKGROUNDCOLOR "jcurzez/BackgroundColor"


#define SIG_COLOR         "Ljcurzez/Color;"
#define SIG_F_COLOR       "Ljcurzez/ForegroundColor;"
#define SIG_B_COLOR       "Ljcurzez/BackgroundColor;"


#define NB_COLORS 8


#define JCURZEZ_START_COLOR_ERR ((jint) -2)
#define JCURZEZ_INITSCR_ERR     ((jint) -1)
#define JCURZEZ_INITSCR_OK      ((jint)  0)
#define JCURZEZ_NO_COLORS       ((jint)  1)


MAPPING mapping[NB_COLORS] = {
  { COLOR_BLACK, "BLACK" },
  { COLOR_RED, "RED" },
  { COLOR_GREEN, "GREEN" },
  { COLOR_YELLOW, "YELLOW" },
  { COLOR_BLUE, "BLUE" },
  { COLOR_MAGENTA, "MAGENTA" },
  { COLOR_CYAN, "CYAN" },
  { COLOR_WHITE, "WHITE" }
};

static jclass    cForegroundColor          = 0;
static jclass    cBackgroundColor          = 0;
static jclass    cColor                    = 0;
static jclass    cCursesColor              = 0;
static jclass    cPen                      = 0;
static jmethodID mIDPenGetBackgroundColor  = 0;
static jmethodID mIDPenGetForegroundColor  = 0;
static jmethodID mIDColorGetIndex          = 0;
static jmethodID mIDCursesColorConstructor = 0;
static jmethodID mIDColorSetRealColor      = 0;
static jmethodID mIDColorGetRealColor      = 0;
static int       hasColors                 = 1;


/********************************************************************************
 * Pure C Functions.
 */

static void set_color(JNIEnv *e, MAPPING m) {
  jfieldID fIDForegroundColor, fIDBackgroundColor;
  jobject oNewColor, oFore, oBack;
  /* The foreground and background color at once. */
  short r, g, b;

  /* Retrieves the content of this color. */
  color_content(m.index, &r, &g, &b);

  /* Retrieves the fields IDs for foreground and background colors. */
  fIDForegroundColor = (*e)->GetStaticFieldID(e, cForegroundColor,
					      m.name,
					      SIG_F_COLOR);

  fIDBackgroundColor = (*e)->GetStaticFieldID(e, cBackgroundColor,
					      m.name,
					      SIG_B_COLOR);

  /* Constructs a new CursesColor. */
  oNewColor =
    (*e)->NewObject(e, cCursesColor, mIDCursesColorConstructor,
		    (jshort) m.index, (jshort) r, (jshort) g, (jshort) b);

  oFore     =
    (*e)->GetStaticObjectField(e, cForegroundColor, fIDForegroundColor);
  oBack     =
    (*e)->GetStaticObjectField(e, cBackgroundColor, fIDBackgroundColor);

  /* Sets both foreground and background values. */
  (*e)->CallVoidMethod(e, oFore, mIDColorSetRealColor, oNewColor);
  (*e)->CallVoidMethod(e, oBack, mIDColorSetRealColor, oNewColor);
}


static void  init_colors(JNIEnv *e) {
  int i;
  short j, k;

  cColor           = (*e)->FindClass(e, C_COLOR);
  cCursesColor     = (*e)->FindClass(e, C_CURSESCOLOR);
  cForegroundColor = (*e)->FindClass(e, C_FOREGROUNDCOLOR);
  cBackgroundColor = (*e)->FindClass(e, C_BACKGROUNDCOLOR);

  mIDCursesColorConstructor =
    (*e)->GetMethodID(e, cCursesColor, "<init>", "(SSSS)V");
  mIDColorSetRealColor      =
    (*e)->GetMethodID(e, cColor, "setRealColor", "(Ljava/lang/Object;)V");

  /* Initializes pairs. */
  for (j = 0; j <= COLORS; j++) {
    for (k = 0; k <= COLORS; k++) {
      int p = j + (k << 3);
      if (p == 0 || p > COLOR_PAIRS - 1)
	continue;
      init_pair(p, j, k);
    }
  }

  /* Initializes colors. */
  for (i = 0; i < NB_COLORS; i++) {
    set_color(e, mapping[i]);
  }
}


/********************************************************************************
 * Static functions
 */
JNIEXPORT jint JNICALL
Java_jcurzez_peer_curses_PeerScreen__1begin(JNIEnv *env, jclass c)
{
  if (initscr() == NULL)
    return JCURZEZ_INITSCR_ERR;
  if (has_colors() == TRUE) {
    if (start_color() != OK) {
      endwin();
      return JCURZEZ_START_COLOR_ERR;
    } else {
      /*Initializes colors + pairs.*/
      init_colors(env);
      return JCURZEZ_INITSCR_OK;
    }
  } else {
    fprintf(stderr, "curses returned: No colors !");
    getchar();
    hasColors = 0;
    return JCURZEZ_NO_COLORS;
  }
}

JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1gotoXY(JNIEnv *e, jclass c,
					     jint x, jint y)
{
  move((int) y, (int) x);
  refresh();
}


JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1printCell(JNIEnv *e,
						jclass c,
						jint x,
						jint y,
						jshort index_fore,
						jshort index_back,
						jbyte attribute,
						jint character)
{
  int attr = (int) attribute;

  chtype ch = (chtype) character;

  switch(ch) {
  case jcurzez_BoxDrawing_LIGHT_HORIZONTAL:
  case jcurzez_BoxDrawing_DOUBLE_HORIZONTAL:
  case jcurzez_BoxDrawing_HEAVY_HORIZONTAL:
    ch = ACS_HLINE;
    break;
  case jcurzez_BoxDrawing_LIGHT_VERTICAL:
  case jcurzez_BoxDrawing_DOUBLE_VERTICAL:
  case jcurzez_BoxDrawing_HEAVY_VERTICAL:
    ch = ACS_VLINE;
    break;
  case jcurzez_BoxDrawing_LIGHT_DOWN_AND_RIGHT:
  case jcurzez_BoxDrawing_DOWN_LIGHT_AND_RIGHT_HEAVY:
  case jcurzez_BoxDrawing_DOWN_HEAVY_AND_RIGHT_LIGHT:
  case jcurzez_BoxDrawing_HEAVY_DOWN_AND_RIGHT:
    ch = ACS_ULCORNER;
    break;
  case jcurzez_BoxDrawing_LIGHT_UP_AND_RIGHT:
  case jcurzez_BoxDrawing_UP_LIGHT_AND_RIGHT_HEAVY:
  case jcurzez_BoxDrawing_UP_HEAVY_AND_RIGHT_LIGHT:
  case jcurzez_BoxDrawing_HEAVY_UP_AND_RIGHT:
    ch = ACS_LLCORNER;
    break;
  case jcurzez_BoxDrawing_LIGHT_UP_AND_LEFT:
  case jcurzez_BoxDrawing_UP_LIGHT_AND_LEFT_HEAVY:
  case jcurzez_BoxDrawing_UP_HEAVY_AND_LEFT_LIGHT:
  case jcurzez_BoxDrawing_HEAVY_UP_AND_LEFT:
    ch = ACS_LRCORNER;
    break;
  case jcurzez_BoxDrawing_LIGHT_DOWN_AND_LEFT:
  case jcurzez_BoxDrawing_DOWN_LIGHT_AND_LEFT_HEAVY:
  case jcurzez_BoxDrawing_DOWN_HEAVY_AND_LEFT_LIGHT:
  case jcurzez_BoxDrawing_HEAVY_DOWN_AND_LEFT:
    ch = ACS_URCORNER;
    break;
  }

  if (hasColors) {
    ch |= (chtype) COLOR_PAIR((short) index_fore +
			      (((short) index_back) << 3));
  }

  /*compute the right attribute.
    A_NORMAL is 0. */
  if (attr != 0) {
    if (attr & 1)
      ch |= A_BLINK;
    if (attr & 2)
      ch |= A_BOLD;
    if (attr & 4)
      ch |= A_DIM;
    if (attr & 8)
      ch |= A_REVERSE;
    if (attr & 16)
      ch |= A_STANDOUT;
    if (attr & 32)
      ch |= A_UNDERLINE;
  }

  mvaddch((int) y, (int) x, ch);
}


JNIEXPORT jint JNICALL
Java_jcurzez_peer_curses_PeerScreen__1getColumns(JNIEnv *env, jclass c)
{
  int maxX, maxY;
  getmaxyx(stdscr, maxY, maxX);
  return maxX;
}

JNIEXPORT jint JNICALL
Java_jcurzez_peer_curses_PeerScreen__1getLines(JNIEnv *env, jclass c)
{
  int maxX, maxY;
  getmaxyx(stdscr, maxY, maxX);
  return maxY;

}


JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1update(JNIEnv *env, jclass c)
{
  refresh();
}

JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1setCursorVisibility(JNIEnv *e, jclass c,
							  jint v)
{
  curs_set((int) v);
}


JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1beep(JNIEnv *env, jclass c)
{
  beep();
}


JNIEXPORT void JNICALL
Java_jcurzez_peer_curses_PeerScreen__1end(JNIEnv *env, jclass c)
{
  endwin();
}
