/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;


/**
 * A <code>Cell</code> contains a character and its decoration.
 * This decoration is: <ul>
 * <li>a {@link #foreground foreground color}</li>
 * <li>a {@link #background background color}</li>
 * <li>an {@link #attribute attribute}</li>
 * </ul>
 * This class is heavily used in {@link jcurzez.AbstractWindow AbstractWindow},
 * as a buffer. Thus all its member are public. And almost all its methods are
 * package private.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class Cell {
    /** Singleton null cell object. */
    public static final Cell NULL_CELL = new Cell(0, null, null, null);

    /** The character to display. */
    public int character;

    /**
     * Indicates if the cell has changed.
     * If so, it requires to be refreshed.
     */
    public boolean hasChanged;

    /**
     * Attribute of the cell.
     *
     * @see       jcurzez.Attribute Attribute
     */
    public Attribute attribute;

    /**
     * Background color of this cell.
     *
     * @see       jcurzez.BackgroundColor BackgroundColor
     */
    public BackgroundColor background;

    /**
     * Foreground color of this cell.
     *
     * @see       jcurzez.ForegroundColor ForegroundColor
     */
    public ForegroundColor foreground;


    /**
     * Copy constructor.
     *
     * @param     source the cell to copy.
     */
    Cell(Cell source) {
	this.character  = source.character;
	this.attribute  = source.attribute;
	this.background = source.background;
	this.foreground = source.foreground;
	this.hasChanged = source.hasChanged;
    }

    /**
     * Creates a new cell.
     *
     * @param     character the character in this cell.
     * @param     attribute the attribute to use to paint this cell.
     * @param     background the attribute to use to paint this cell.
     * @param     foreground the attribute to use to paint this cell.
     * @param     changed <code>true</code> if this cell has changed.
     */
    Cell(int character, Attribute attribute,
	 BackgroundColor background,
	 ForegroundColor foreground,
	 boolean changed) {
	this.character  = character;
	this.attribute  = attribute;
	this.background = background;
	this.foreground = foreground;
	this.hasChanged = changed;
    }

    /**
     * Creates a new cell with specified decoration and specified character.
     *
     * @param     character the character to use in this cell.
     * @param     attribute the attribute to use to paint this cell.
     * @param     background the attribute to use to paint this cell.
     * @param     foreground the attribute to use to paint this cell.
     */
    Cell(int character, Attribute attribute,
	 BackgroundColor background,
	 ForegroundColor foreground) {
	this(character, attribute, background, foreground, false);
    }

    /**
     * Copies <code>this</code> to <code>destinationCell</code>.
     *
     * @param     destinationCell the cell where to copy <code>this</code>.
     */
    void copyInto(Cell destinationCell) {
	destinationCell.character  = character;
	destinationCell.attribute  = attribute;
	destinationCell.background = background;
	destinationCell.foreground = foreground;
	destinationCell.hasChanged = true;
    }

    /**
     * Copies into this cell given attributes.
     *
     * @param     character the character to use in this cell.
     * @param     attribute the attribute to use to paint this cell.
     * @param     background the attribute to use to paint this cell.
     * @param     foreground the attribute to use to paint this cell.
     */
    void copyFrom(int character, Attribute attribute,
		  BackgroundColor background,
		  ForegroundColor foreground) {
	this.character  = character;
	this.attribute  = attribute;
	this.background = background;
	this.foreground = foreground;
	this.hasChanged = true;
    }

    /**
     * Returns <code>true</code> if <code>this</code> and <code>cell</code>
     * have the same decoration. That is to say foreground color, background
     * color and attribute.
     *
     * @param      cell the cell to compare <code>this</code> to.
     * @return     <code>true</code> if <code>cell</code> and <code>this</code>
     *             have the same decoration.
     */
    public boolean hasSameDecoration(Cell cell) {
	return
	    (this.attribute.equals(cell.attribute))  &&
	    (this.background == cell.background)     &&
	    (this.foreground == cell.foreground);
    }

    /**
     * Returns a string representation of <code>this</code>, in the following
     * form: <code>[ <em>foregorund</em> on <em>background</em>,
     * <em>attribute</em> ]</code>.
     */
    public String toString() {
	StringBuffer sb = new StringBuffer("[ ");
	sb.append(character).append(' ');
	sb.append(foreground);
	sb.append(" on ").append(background).append(", ");
	sb.append(attribute).append(" ]");
	return sb.toString();
    }
}
